import unittest
import sys
import pytest
import sidpy
from pywget import wget
import os
import pickle
import numpy as np

sys.path.append("../../../../../SciFiReaders/")
import SciFiReaders as sr

root_path = "https://github.com/pycroscopy/SciFiDatasets/blob/main/data/microscopy/spm/afm/"

igor2 = pytest.importorskip("igor2", reason="igor2 not installed")

class TestIgorIBW(unittest.TestCase):
    # Tests the nanonis_dat reader
    def test_igor_matrix_file_cits(self):
        #Test the CITS file reads correctly
        file_cits = '20230110-152047_ScV6Sn6-2023-01-10-STM_NANOPROBE_AtomManipulation--37_1-Aux2(V) 3_Forward_Down_Trace.ibw?raw=true'
        file_path = os.path.join(root_path, file_cits)
        file_out = 'cits_file.ibw'
        wget.download(file_path, out=file_out)
        reader = sr.IgorMatrixReader(file_out)
        dataset = reader.read()

        #a lot of asserts here
        assert dataset.data_type == sidpy.DataType.SPECTRAL_IMAGE, \
        "Expected data type {}, received {}".format(sidpy.DataType.SPECTRAL_IMAGE,dataset.data_type )

        assert dataset.ndim==3, "Dataset dimension should be 3, received {} instead".format(dataset.ndim)
        true_dd = 'Aux2(V) 3 37-1 Forward Down Trace (V)'
        assert dataset.data_descriptor==true_dd, "Data descriptor was expected to be {} but received {}".format(true_dd, dataset.data_descriptor)
        dimension_sizes = [64,8,256]
        dimension_types = [sidpy.DimensionType.SPATIAL,sidpy.DimensionType.SPATIAL,
                           sidpy.DimensionType.SPECTRAL]
        for dim in dataset._axes:
            dimension = dataset._axes[dim]
            assert dimension.size == dimension_sizes[dim]
            assert dimension.dimension_type == dimension_types[dim]

        #Load the metadata
        file_metadata = 'orig_dict.p?raw=true'
        file_path = os.path.join(root_path, file_metadata)
        metadata_out = 'cits_metadata.p'
        wget.download(file_path, out=metadata_out)
        true_metadata = pickle.load(open(metadata_out, 'rb'))
        received_metadata = dataset.original_metadata
        for key in true_metadata: 
            if type(true_metadata[key])==np.ndarray:
                assert true_metadata[key].all()==received_metadata[key].all(), \
                "Was expecting {} for key {} but received {}".format(true_metadata[key],
                                            key,received_metadata[key] )
            elif type(true_metadata[key]) != dict:
                assert true_metadata[key]==received_metadata[key], \
                "Was expecting {} for key {} but received {}".format(true_metadata[key],
                                            key,received_metadata[key] )

        os.remove(file_out)
        os.remove(metadata_out)
        return
    
    def test_igor_matrix_file_image(self):
        #Test the image file reads correctly
        file_cits = '20230110-152047_ScV6Sn6-2023-01-10-STM_NANOPROBE_AtomManipulation--37_1-I 3_Backward_Down.ibw?raw=true'
        file_path = os.path.join(root_path, file_cits)
        file_out = 'img_file.ibw'
        wget.download(file_path, out=file_out)
        reader = sr.IgorMatrixReader(file_out)
        dataset = reader.read()

        #a lot of asserts here
        assert dataset.data_type == sidpy.DataType.IMAGE, \
        "Expected data type {}, received {}".format(sidpy.DataType.IMAGE,dataset.data_type )

        assert dataset.ndim==2, "Dataset dimension should be 3, received {} instead".format(dataset.ndim)
        true_dd = 'I 3 37-1 Backward Down (A)'
        assert dataset.data_descriptor==true_dd, "Data descriptor was expected to be {} \
        but received {}".format(true_dd, dataset.data_descriptor)
        dimension_sizes = [58,256]
        dimension_types = [sidpy.DimensionType.SPATIAL, sidpy.DimensionType.SPATIAL]
        for dim in dataset._axes:
            dimension = dataset._axes[dim]
            assert dimension.size == dimension_sizes[dim]
            assert dimension.dimension_type == dimension_types[dim]

        #Load the metadata
        file_metadata = 'image_dict.p?raw=true'
        file_path = os.path.join(root_path, file_metadata)
        img_metadata_out = 'img_metadata.p'
        wget.download(file_path, out=img_metadata_out)
        true_metadata = pickle.load(open(img_metadata_out, 'rb'))
        received_metadata = dataset.original_metadata
        for key in true_metadata: 
            if type(true_metadata[key])==np.ndarray:
                assert true_metadata[key].all()==received_metadata[key].all(), \
                "Was expecting {} for key {} but received {}".format(true_metadata[key],
                                            key,received_metadata[key] )
            elif type(true_metadata[key]) != dict:
                assert true_metadata[key]==received_metadata[key], \
                "Was expecting {} for key {} but received {}".format(true_metadata[key],
                                            key,received_metadata[key] )

        os.remove(file_out)
        os.remove(img_metadata_out)

    
    def test_load_test_ibw_force_file(self):

        # Test if the test dat file can be read in correctly

        file_path = 'force_ibw.ibw'
        # Download the required files
        wget.download(root_path + "/IgorIBWReader_ForceCurve.ibw?raw=true", out=file_path)

        data_translator = sr.IgorIBWReader(file_path)
        datasets = data_translator.read(verbose=False)
        assert len(datasets)==3, "Length of dataset should be 3 but is instead {}".format(len(datasets))
        metadata = datasets['Channel_000'].original_metadata
        data_descriptors = ['Raw (m)', 'Defl (m)', 'ZSnsr (m)']

        original_metadata ={'MostPosZVoltage': 150,
        'ARDFForceMap': 1,
        'NumOfSegments': 3,
        'Direction': 'Nan,1,-1,0,',
        'VerDate': 130100,
        'Version': '13.16.100',
        'XOPVersion': '20122.33.35.0',
        'OSVersion': 'Windows 7 Professional Service Pack 1 (Build 7601)',
        'IgorFileVersion': '6.3.2.3',
        'XLVDT': 3.1665e-06,
        'YLVDT': -4.1177e-08,
        'ARDoIVCurve': 0,
        'ForceNote': '',
        'TipSerialNumber': '',
        'ForceMapNote': '',
        'MostNegZvoltage': -10,
        'ExtendZ': 1.0915e-05,
        'RetractZ': -7.277e-07,
        'StartDist': 7.1484e-06,
        'ForceDist': 2e-07,
        'ForceDistSign': 1,
        'VelocitySynch': 1,
        'Velocity': 3.9683e-07,
        'ApproachVelocity': 1.9841e-06,
        'RetractVelocity': 1.9841e-06,
        'ForceScanRate': 0.99206,
        'ApproachScanRate': 1,
        'RetractScanRate': 1,
        'UseVelocity': 0,
        'DwellSetting': 0,
        'DwellTime': 0.992,
        'DwellTime1': 0.992,
        'DwellRate': 10,
        'HamSensPercent': 1,
        'HamSens': 1e-06,
        'HamWhich': 0,
        'LastKnownZDist': 1e-06,
        'UsePerDist': 0,
        'ForceFilterBW': 1000,
        'NumPtsPerSec': 2000,
        'NumPtsPerNm': 1,
        'TarPtsPerNm': 1,
        'NumPtsPerWave': 2000,
        'TarPtsPerWave': 2000,
        'MaxPtsPerSec': 100000,
        'ContForce': 2,
        'ZStateChanged': 0,
        'UseStartDist': 0,
        'OldStartDist': 3.0724e-06,
        'OldForceDist': 1e-06,
        'SaveForce': 3,
        'SaveWhat': 1,
        'ForceSpotNumber': 3,
        'ForceCrosspointChange': 0,
        'TriggerChannel': 'DeflVolts',
        'TriggerSlope': 1,
        'TriggerPoint': 0.15,
        'TriggerType': 0,
        'MaxContinuousForce': 'inf',
        'ContinuousForceCounter': 0,
        'ShowXYSpot': 3,
        'TriggerNone': 0.2,
        'TriggerDeflection': 1e-07,
        'TriggerAmplitude': 8.72e-08,
        'TriggerPhase': 65,
        'TriggerUserIn0': 0,
        'TriggerUserIn1': 0,
        'TriggerUserIn2': 0,
        'TriggerLateral': 0,
        'TriggerFrequency': 0,
        'TriggerCurrent': 0,
        'TriggerCurrent2': 0,
        'TriggerCurrentVolts': 0,
        'TriggerCurrent2Volts': 0,
        'TriggerCount': 0,
        'TriggerInputI': 0.2,
        'TriggerInputQ': 1.235,
        'TriggerForce': 1.8189e-09,
        'TriggerRawZSensor': 1e-06,
        'TriggerDrive': 0,
        'TriggerDriveVolts': 0,
        'TriggerDeflVolts': 1,
        'TriggerAmpVolts': 0.8,
        'TriggerZSensorVolts': 0.76464,
        'TriggerAmpPercent': 90,
        'InputTime': 11.008,
        'FakeCont': 0,
        'KappaFactor': 1.09,
        'ForceMode': 'Open Loop',
        'VirtDeflSlope': 0,
        'VirtDeflOffset': 0,
        'DisplayVirtDeflSlope': 0,
        'LastUsedVirtDeflSlope': 0,
        'LastUsedVirtDeflOffset': 0,
        'UsingVirtDefl': 0,
        'AllowForceNoteUpdate': 0,
        'IndentMode': 0,
        'DwellRampMode': 'Displacement',
        'DwellRampParm0': 1e-07,
        'DwellRampParm1': 1e-07,
        'DwellRampParm2': 1e-07,
        'DwellRampParm3': 1,
        'DwellRampFunc': 'ARIndentTriangle',
        'DwellRampUnits': 1,
        'VirtDefl2ndTerm': 0,
        'ForceDecimation': 1,
        'TriggerUserIn0Volts': 0,
        'triggerUserIn1Volts': 0,
        'TriggerUserIn2Volts': 0,
        'UseVirtDeflOffset': 0,
        'TriggerAmplitude1': 8.72e-08,
        'TriggerPhase1': 65,
        'TriggerAmplitude2': 3e-08,
        'TriggerPhase2': 65,
        'UsePolyInvols': 0,
        'PolyInvols4': 0,
        'PolyInvols3': 0,
        'PolyInvols2': 0,
        'InvolsBySum': 0,
        'TriggerAmp1Volts': 0.8,
        'TriggerAmp2Volts': 0.06,
        'FMapScanTime': 17.451,
        'FMapScanPoints': 32,
        'FMapScanLines': 32,
        'FMapDelayUpdate': 1,
        'FMapSlowScanDisabled': 0,
        'FMapDisplayLVDTTraces': 0,
        'FMapStatus': 1,
        'FMapCounter': 1,
        'FMapXYVelocity': 2e-05,
        'FMapScanDown': 0,
        'FMapIsXYMax': 0,
        'FMapTimeHasForceRate': 1,
        'FMapBaseSuffix': 0,
        'FMapLineCounter': 0,
        'FMapPointCounter': 0,
        'UseDwellRate': 0,
        'DwellFilterRate': 5,
        'RTForceStartIndex': 35065,
        'RTForceSkipFactor': 1,
        'UpdateTriggerTime': 0,
        'UpdateDidTrigger': 0,
        'UpdateWrittenPoints': 0,
        'DoRTForceUpdate': 1,
        'ARDoIVFP': 0,
        'UsingIndenting': 0,
        'FMapContinuous': 0,
        'MaxDwellPoints': 1000000,
        'FMapPaused': 0,
        'Force32BitChannel': 0,
        'FMapBookwise': 0,
        'FMapSave': 2,
        'ForceSaveList': 'Deflection;ZSensor;Force;DeflVolts;ZSensorVolts;RawZSensor;',
        'ForceDisplay0': 'Deflection;',
        'ForceDisplay1': '',
        'ForceDisplay2': '',
        'ForceDisplay3': '',
        'ForceDisplay4': '',
        'ForceGraphNumber': 2,
        'AutoRemoveFit': 1,
        'TriggerIndentation': 0,
        'RelativeNone': 1,
        'RelativeDeflection': 1.5e-08,
        'RelativeAmplitude': 2.18e-08,
        'RelativePhase': 65,
        'RelativeUserIn0': 0,
        'RelativeUserIn1': 0,
        'RelativeUserIn2': 0,
        'RelativeLateral': 0,
        'RelativeFrequency': 0,
        'RelativeCurrent': 0,
        'RelativeCurrent2': 0,
        'RelativeCurrentVolts': 0,
        'RelativeCurrent2Volts': 0,
        'RelativeCount': 0,
        'RelativeInputI': 0.2,
        'RelativeInputQ': 1.235,
        'RelativeForce': 2.7284e-10,
        'RelativeRawZSensor': 1,
        'RelativeDrive': 0,
        'RelativeDriveVolts': 0,
        'RelativeDeflVolts': 0.15,
        'RelativeAmpVolts': 0.2,
        'RelativeZSensorVolts': 19,
        'RelativeAmpPercent': 90,
        'RelativeUserIn0Volts': 0,
        'RelativeUserIn1Volts': 0,
        'RelativeUserIn2Volts': 0,
        'RelativeAmplitude1': 8.72e-08,
        'RelativePhase1': 65,
        'RelativeAmplitude2': 3e-08,
        'RelativePhase2': 65,
        'RelativeAmp1Volts': 0.8,
        'RelativeAmp2Volts': 0.06,
        'RelativeIndentation': 0,
        'AutoForceChannel': 1,
        'AdvancedTriggers': 0,
        'ForceChannelPanelControls': 0,
        'ForceChannelPanelLessControls': 0,
        'ForceChannelPanelSimpleControls': 0,
        'GoThereWithdraw': 0,
        'ZFilterBW': 2000,
        'IsTriggered': 1,
        'InA': 'Ground',
        'InB': 'Ground',
        'InFast': 'FilterOut',
        'InAOffset': 'Ground',
        'InBOffset': 'Ground',
        'InFastOffset': 'Ground',
        'OutXMod': 'Off',
        'OutYMod': 'Off',
        'OutZMod': 'Off',
        'FilterIn': 'Defl',
        'BNCOut0': 'Ground',
        'BNCOut1': 'Ground',
        'BNCOut2': 'Ground',
        'PogoOut': 'Ground',
        'Chip': 'Ground',
        'Shake': 'DDS',
        'ScanSize': 9e-06,
        'FastScanSize': 9e-06,
        'SlowScanSize': 9e-06,
        'ScanRate': 0.89799,
        'ScanSpeed': 2.0205e-05,
        'XOffset': -2.1179e-06,
        'YOffset': -5.5288e-06,
        'RoundFactor': 0.04,
        'IntegralGain': 34.235,
        'ProportionalGain': 0,
        'ScanAngle': 336,
        'ScanAngleFactor': 1,
        'AmplitudeSetpointVolts': 0.8,
        'AmplitudeSetpointMeters': 1e-08,
        'DriveAmplitude': 0.80231,
        'DriveFrequency': 6629.5,
        'SweepWidth': 6000,
        'SlowScanEnabled': 1,
        'DeflectionSetpointVolts': 1,
        'DeflectionSetpointMeters': 0,
        'DeflectionSetpointNewtons': 5e-09,
        'ImagingMode': 'Contact',
        'MaxScanSize': 9e-05,
        'InvOLS': 1e-07,
        'SpringConstant': 0.018189,
        'DisplaySpringConstant': 'Nan',
        'ScanStateChanged': 1,
        'ScanDown': 0,
        'XLVDTSens': 8.1359e-06,
        'YLVDTSens': 7.766e-06,
        'ZLVDTSens': 1.3002e-06,
        'XLVDTOffset': 0.26,
        'YLVDTOffset': 0.75,
        'ZLVDTOffset': 0,
        'YIgain': 3.79,
        'YPgain': '-inf',
        'XIgain': 3.86,
        'XPgain': '-inf',
        'LastScan': 0,
        'XPiezoSens': 5.202e-07,
        'YPiezoSens': 4.8789e-07,
        'ZPiezoSens': 7.277e-08,
        'XDriveOffset': 0,
        'YDriveOffset': 0,
        'SweepPoints': 480,
        'SecretGain': 0,
        'YSgain': 5.79,
        'XSgain': 5.81,
        'DIsplayLVDTTraces': 0,
        'PhaseOffset': 149.1,
        'BaseSuffix': 17,
        'SaveImage': 2,
        'ParmChange': 0,
        'ADCgain': 0,
        'OldADCgain': 12,
        'OldScanSize': 'NaN',
        'OldYOffset': -5.5288e-06,
        'OldXOffset': -2.1179e-06,
        'ZIgain': 4.26,
        'ZPgain': '-inf',
        'ZSgain': 0,
        'AmpInvOLS': 1.09e-07,
        'UpdateCounter': 106,
        'DoMunge': 0,
        'XMungeAlpha': 1,
        'YMungeAlpha': 1,
        'ZMungeAlpha': 1,
        'Has1xACDeflGain': 1,
        'DontChangeXPT': 0,
        'LowNoise': 1,
        'ScreenRatio': 0.75,
        'XDriftRate': 0,
        'YDriftRate': 0,
        'XDriftTotal': 0,
        'YDriftTotal': 0,
        'DriftBIts': 0,
        'DriftCount': 0,
        'Setpoint': 0.0008,
        'FastRatio': 1,
        'SlowRatio': 1,
        'TopLine': 255,
        'BottomLine': 0,
        'ScanStatus': 0,
        'Is1DPlus': 0,
        'DelayUpdate': 0,
        'MarkerRatio': 0,
        'StartLineCount': 0,
        'OldAmplitudeSetpointVolts': 0.18552,
        'OldDeflectionSetPointVolts': 0,
        'ScanMode': 0,
        'Interpolate': -1248,
        'TipVoltage': 0,
        'SurfaceVoltage': 0,
        'FreqIgain': 5,
        'FreqPgain': 0,
        'FreqSgain': 0,
        'OrcaGain': 'NaN',
        'FreqGainOn': 0,
        'UserIn0Gain': 1,
        'UserIn1Gain': 1,
        'UserIn2Gain': 1,
        'LateralGain': 1,
        'IsBipolar': 0,
        'FrequencyHack': 0,
        'MagneticField': 'NaN',
        'DriveGainOn': 0,
        'DriveIGain': 50,
        'DrivePGain': 0,
        'DriveSGain': 0,
        'UserIn0Offset': 0,
        'UserIn1Offset': 0,
        'UserIn2Offset': 0,
        'LateralOffset': 0,
        'LastImage': 0,
        'ZoomSize': 9.5217e-06,
        'ZoomXOffset': -2.1179e-06,
        'ZoomYOffset': -5.5288e-06,
        'User0Voltage': 0,
        'User1Voltage': 0,
        'ExtendedZ': 0,
        'DriveAmplitude1': 0.1,
        'DriveFrequency1': 450000,
        'SweepWidth1': 5000,
        'PhaseOffset1': 0,
        'Amp2InvOLS': 2e-08,
        'VoltageParm': 0,
        'MicroscopeID': 3,
        'CapacitanceSens': 1,
        'CapacitanceOffset': 0,
        'CapPhaseOffset': 0,
        'TipBiasOffset': 0,
        'TipHeaterOffset': 0,
        'SurfaceBiasOffset': 0,
        'UseCantHolder': 0,
        'CurrentCantHolder': 2,
        'HasClickRing': 1,
        'XScanDirection': 1,
        'YScanDirection': 1,
        'SaveImageCount': 0,
        'SaveImageLines': 128,
        'OrcaGain2': 'NaN',
        'UpdateSlowWave': 0,
        'FBFilterBW': 1500,
        'CurrentSetpointAmps': 1,
        'CurrentSetpointVolts': 1,
        'OldCurrentSetpointVolts': 1,
        'OldCurrentSetpointAmps': 1,
        'HasFM': 1,
        'LastScanWithdraw': 0,
        'HasLaserRelay': 1,
        'ToggleLaserRelay': 0,
        'Decimation': 87,
        'DARTIGain': 5.5,
        'DARTPGain': 0,
        'SampleHolder': 1,
        'ManualSampleHolder': 0,
        'CypherXPTLock': 0,
        'XPTLock': 0,
        'LogFeedback': 0,
        'ARSysScanBusy': 0,
        'SimpleControls': 0,
        'FreeAirAmplitude': 0,
        'FreeAirPhase': 0,
        'FreeAirDeflection': 0,
        'FreeAirDriveAmplitude': 0,
        'ContinualRetune': 0,
        'ForceFilter': 0,
        'TemperatureSens': 1,
        'TemperatureOffset': 0,
        'OffsetBehavior': 0,
        'InputRange': 10,
        'XLVDTSensSlope': 0,
        'YLVDTSensSlope': 0,
        'XLVDTSensIntercept': 0,
        'YLVDTSensIntercept': 0,
        'HaveWeKilledTheProgressBar': 1,
        'PreScanSetting': 0,
        'ImageFrameTime': 0,
        'FBFilterSelectivity': 1,
        'HasDriveChoke': 0,
        'DriveChokeOn': 0,
        'CFCWFOS': 0,
        'SkewAngle': 0,
        'ContinualRetuneTime': 60,
        'DRTimeStamp': 0,
        'CRTimeStamp': 0,
        'Cypher15VPower': 0,
        'HasBlueDrive': 0,
        'BlueDriveOn': 0,
        'blueDriveOffset': 5,
        'blueDriveMaxAmplitude': 0,
        'bDDriveAmplitude': 0.002,
        'bDDriveAmplitude1': 0.002,
        'bDDiffEfficiencyV': 0.004475,
        'bdReCalibrateTime': 0,
        'bdMotorError': 0,
        'ScanModeGUI': 0,
        'FastMapScanPoints': 256,
        'FastMapScanLines': 256,
        'FastMapOptScanPoints': 256,
        'FastMapOptScanLines': 256,
        'FreqDGain': 0,
        'DriveDGain': 0,
        'ImageSaveExpand': 0,
        'Saturated': 0,
        'LastThreadHandle': 4817,
        'MicroscopeModel': 'MFP3D',
        'ThermalDC': 1e-14,
        'ThermalQ': 20,
        'ThermalFrequency': 75000,
        'ThermalWhiteNoise': 5e-15,
        'ScaleLog': 3,
        'FitWidth': 20000,
        'ThermalSamples': 1000,
        'ThermalCounter': 0,
        'SectionSamples': 10,
        'SectionCounter': 0,
        'DoThermal': 0,
        'ThermalResolution': 4,
        'ThermalZoom': 0,
        'DoCantTune': 0,
        'ZoomCounter': 200,
        'AutoTuneLow': 50000,
        'AutoTuneHigh': 400000,
        'TargetVoltage': 1,
        'TargetPercent': 0,
        'CheckTuneTraces': 0,
        'ButtonTime': 6561100000000,
        'ShowThermalFit': 0,
        'AppendThermalBit': 8,
        'TuneQ': 100,
        'TuneGain': 0,
        'TunePeakResult': 0.87327,
        'TuneFreqResult': 9340.3,
        'TuneQResult': 'NaN',
        'ThermalSamplesLimit': 0,
        'TuneFeedback': 0,
        'TuneDDSOutput': 0,
        'TipVoltCenter': 0,
        'TipVoltWidth': 10,
        'ThermalFrequencyRange': 1,
        'AutoTuneLimit': 0,
        'AutoTuneLow1': 300000,
        'AutoTuneHigh1': 600000,
        'TargetVoltage1': 0.1,
        'TargetPercent1': 0,
        'DualACMode': 0,
        'WhichACMode': 0,
        'BackwardsTune': 0,
        'TuneCaptureRate': 500,
        'TuneTime': 0.96,
        'TuneFilter': 500,
        'iDriveOn': 0,
        'FrequencyRatio': 6,
        'DFRTFrequencyCenter': 262500,
        'DFRTFrequencyWidth': 375000,
        'DFRTOn': 0,
        'ThermalCenter': 75000,
        'ThermalWidth': 30000,
        'DFWMAMT': 0,
        'LastXPos': 0,
        'LastYPos': 0,
        'PFMFrequencyLimit': 'inf',
        'TuneLockin': 0,
        'TipHeaterDrive': 0,
        'AdjustSpringConstant': 1,
        'DrawThermalFit': 0,
        'LateralTune': 0,
        'ThermalTemperature': 298,
        'UseCypherThermal': 0,
        'UseCypherMultiThread': 0,
        'Illumination': 0,
        'ARCZ': 0,
        'TuneAngle': -90,
        'IsDefaultFrequency': 1,
        'TargetAmplitude': 0,
        'TargetPhase': 90,
        'LossTanAmp1': 0.55042,
        'LossTanPhase1': 0.44495,
        'LossTanAmp2': 0.032757,
        'LossTanPhase2': -64.594,
        'LossTanFreq1': 6629.5,
        'LossTanFreq2': 0,
        'LossTanQ1': 'NaN',
        'LossTanQ2': 0,
        'SecondFrequency': 0,
        'TryFitAgain': 0,
        'HighFrequencyMode': 0,
        'ThermalLever': 0,
        'DoSader': 0,
        'CustomLeverLength': 0.00024,
        'CustomLeverWidth': 3e-05,
        'CustomLeverFrequency': 70000,
        'ResFreq1': 0,
        'ResFreq2': 0,
        'GetStartedRoughness': 4,
        'GetStartedOn': 0,
        'GSGainGain': 1,
        'GSSetpointPercent': 50,
        'GetStartedMorePanel': 0,
        'GSHardness': 0,
        'GSLeverRange': 0,
        'GSUseManualThermal': 0,
        'GSMotorHeight': 0,
        'GSStickySample': 0,
        'GSDateTime': 0,
        'BadThermal': 0,
        'UserCalcUnit': '°',
        'UserIn0Unit': 'V',
        'UserIn1Unit': 'V',
        'UserIn2Unit': 'V',
        'LateralUnit': 'V',
        'ARUserPanelName': '',
        'UserIn0Name': '',
        'UserIn1Name': '',
        'UserIn2Name': '',
        'UserCalcName': '',
        'UserIn0Ab': '',
        'UserIn1Ab': '',
        'UserIn2Ab': '',
        'UserCalcAb': '',
        'UserIn0Force': '',
        'UserIn1Force': '',
        'UserIn2Force': '',
        'UserCalcForce': '',
        'NavLoadPath': '',
        'LastNavLoadPath': '',
        'UserCalcBUnit': '',
        'UserCalcBName': '',
        'UserCalcBForce': '',
        'UserCalcBAb': '',
        'OrcaOffset': 0,
        'OrcaOffset2': 0,
        'YourVariablesStartHere': 0,
        'DetectorSum': 'Head.Laser.Sum',
        'XSensor': 'Input.X',
        'YSensor': 'Input.Y',
        'ZSensor': 'Input.Z',
        'DDSAmplitude0': '$Lockin.0.Amp',
        'DDSDCOffset0': '$Lockin.DCOffset',
        'DDSFrequency0': '$Lockin.0.Freq',
        'DDSFrequencyOffset0': '$Lockin.0.FreqOffset',
        'DDSPhaseOffset0': '$Lockin.0.PhaseOffset',
        'DDSAmplitude1': '$Lockin.1.Amp',
        'DDSFrequency1': '$Lockin.1.Freq',
        'DDSPhaseOffset1': '$Lockin.1.PhaseOffset',
        'Amplitude': '$Lockin.0.r',
        'Phase': '$Lockin.0.theta',
        'Inputi': '$Lockin.0.i',
        'Inputq': '$Lockin.0.q',
        'Inputi1': '$Lockin.1.i',
        'Inputq1': '$Lockin.1.q',
        'Count': 'Input.Counter0',
        'Count2': 'Input.Counter1',
        'Potential': '$DDSDCOffset0',
        'Amplitude1': '$Lockin.1.r',
        'Phase1': '$Lockin.1.theta',
        'DDSQGain0': '$Lockin.0.QGain',
        'DDSQAngle0': '$Lockin.0.QAngle',
        'DDSFrequencyOffset1': '$Lockin.1.FreqOffset',
        'Deflection': 'Input.Fast',
        'Lateral': 'Head.Laser.Lateral',
        'Frequency': '$DDSFrequencyOffset0',
        'LockIn': 'ARC.Lockin',
        'Height': 'output.Z',
        'Dissipation': '$DDSAmplitude0',
        'outputXLoop': 'PIDSloop.0',
        'outputYLoop': 'PIDSloop.1',
        'StartHeadTemp': '28.875 °C',
        'StartScannerTemp': '23.5 °C',
        'StartTempSeconds': 3539255155.317,
        'StartHeaterTemp': 'NaN °C',
        'StartHeaterHumidity': 'NaN',
        'EndHeadTemp': 'None Yet',
        'EndScannerTemp': 'None Yet',
        'EndTempSeconds': 0,
        'Indexes': '0,224,1232,1260,',
        'Seconds': 3539255520.083,
        'TriggerTime': 0.1119,
        'HeaterTemperature': 'NaN °C',
        'HeaterHumidity': 'NaN',
        'Invols': 1e-07,
        'AmpInvols': 1.09e-07,
        'Amp2Invols': 2e-08,
        'Locks': '',
        'UserIn3Gain': 1,
        'UserIn3Unit': 'V',
        'UserIn3Force': '',
        'UserIn4Gain': 1,
        'UserIn4Unit': 'V',
        'UserIn4Force': '',
        'UserIn5Gain': 1,
        'UserIn5Unit': 'V',
        'UserIn5Force': '',
        'UserIn6Gain': 1,
        'UserIn6Unit': 'V',
        'UserIn6Force': '',
        'UserIn7Gain': 1,
        'UserIn7Unit': 'V',
        'UserIn7Force': '',
        'UserIn8Gain': 1,
        'UserIn8Unit': 'V',
        'UserIn8Force': '',
        'UserIn9Gain': 1,
        'UserIn9Unit': 'V',
        'UserIn9Force': '',
        'creationDate': 3563961474,
        'modDate': 3563961474,
        'bname': b'Line0010Point0025'}

        for key in original_metadata:
            if original_metadata[key] =='inf' or original_metadata[key] =='-inf': 
                pass
            else:
                assert original_metadata[key] == metadata[key], "Metadata incorrect for key {}, should be {} " \
                        "but was read as {}".format(key, original_metadata[key], metadata[key])

        #for ind in range(len(datasets)):
        for ind, key in enumerate(datasets):
            assert type(datasets[key])== sidpy.sid.dataset.Dataset, "Dataset No. {} not read in as sidpy dataset" \
                    "but was instead read in as {}".format(ind, type(datasets[key]))

            assert datasets[key].shape[0]==1261, "Dataset[{}] is of size 1261 but was read in as {}".format(ind, datasets[key].shape[0])
            assert type(datasets[key]._axes[0]) == sidpy.sid.dimension.Dimension, "Dataset should have dimension type " \
                                           "of sidpy Dimension, but is instead {}".format(type(datasets[key]._axes))
            assert datasets[key].data_descriptor == data_descriptors[ind], "Dataset {} " \
            "should have descriptor {} but instead has descriptor {}".format(ind, data_descriptors[ind], datasets[key].data_descriptor)

        os.remove(file_path)

    def test_load_test_ibw_image_file(self):
        #Test if the IGOR Image IBW file can be read in correctly

        file_path = 'image_ibw.ibw'
        wget.download(root_path + "/IgorIBWReader_ImageStack_BTFO_DSO.ibw?raw=true", out=file_path)

        data_translator = sr.IgorIBWReader(file_path)
        datasets = data_translator.read(verbose=True)
        assert len(datasets)==4, "Length of dataset should be 4 but is instead {}".format(len(datasets))
        data_descriptors = ['HeightRetrace (m)',
        'AmplitudeRetrace (m)',
        'DeflectionRetrace (m)',
        'PhaseRetrace (deg)']
        
        original_metadata = {'ScanSize': 2e-06,
        'FastScanSize': 2e-06,
        'SlowScanSize': 2e-06,
        'ScanRate': 2.0032,
        'XOffset': 0,
        'YOffset': 0,
        'PointsLines': 256,
        'ScanPoints': 256,
        'ScanLines': 256,
        'ScanAngle': 0,
        'ImagingMode': 'PFM Mode',
        'InvOLS': 3.7919e-08,
        'SpringConstant': 0.46538,
        'AmpInvOLS': 4.1332e-08,
        'Amp2Invols': 2e-08,
        'FastRatio': 1,
        'SlowRatio': 1,
        'TopLine': 255,
        'BottomLine': 0,
        'ScanMode': 'Closed Loop',
        'NapMode': 0,
        'FMapScanTime': 17.753,
        'FMapScanPoints': 32,
        'FMapScanLines': 32,
        'FMapXYVelocity': 2e-06,
        'FMapBookWise': 0,
        'Channel1DataType': 'Height',
        'DataTypeSum': 15,
        'PhaseOffset': 0,
        'PhaseOffset1': 0,
        'NapPhaseOffset': 90,
        'VoltageParm': 0,
        'NapParms': 0,
        'PreScanSetting': 0,
        'OrcaOffset': 0,
        'OrcaOffset2': 0,
        'UserIn0Gain': 1,
        'UserIn0Offset': 0,
        'UserIn1Gain': 1,
        'UserIn1Offset': 0,
        'UserIn2Gain': 1,
        'UserIn2Offset': 0,
        'LateralGain': 1,
        'LateralOffset': 0,
        'OrcaGain': 'NaN',
        'OrcaGain2': 'NaN',
        'FastMapScanRate': 0.46781,
        'FastMapZRate': 299.4,
        'BackPackIn0Offset': 0,
        'BackPackIn0Gain': 1,
        'BackPackIn1Offset': 0,
        'BackPackIn1Gain': 1,
        'BackPackIn2Offset': 0,
        'BackPackIn2Gain': 1,
        'BackPackIn3Offset': 0,
        'BackPackIn3Gain': 1,
        'BackPackIn4Offset': 0,
        'BackPackIn4Gain': 1,
        'Real Parms': 'End',
        'Initial Parms': 'End',
        'Initial ScanSize': 2e-06,
        'Initial FastScanSize': 2e-06,
        'Initial SlowScanSize': 2e-06,
        'Initial ScanRate': 2.0032,
        'ScanSpeed': 1.0016e-05,
        'Initial XOffset': 0,
        'Initial YOffset': 0,
        'Initial ScanPoints': 256,
        'Initial ScanLines': 256,
        'RoundFactor': 0.04,
        'IntegralGain': 50,
        'ProportionalGain': 0,
        'Initial ScanAngle': 0,
        'ScanAngleFactor': 1,
        'AmplitudeSetpointVolts': 0.8,
        'AmplitudeSetpointMeters': 1e-08,
        'DriveAmplitude': 1,
        'DriveFrequency': 380740,
        'SweepWidth': 180650,
        'SlowScanEnabled': 1,
        'DeflectionSetpointVolts': 1,
        'DeflectionSetpointMeters': 0,
        'DeflectionSetpointNewtons': 5e-09,
        'Initial ImagingMode': 3,
        'MaxScanSize': 3e-05,
        'Initial InvOLS': 3.7919e-08,
        'Initial SpringConstant': 0.46538,
        'DisplaySpringConstant': 4.6538e-10,
        'ScanStateChanged': 0,
        'ScanDown': 0,
        'XLVDTSens': 3.9412e-06,
        'YLVDTSens': 4.0876e-06,
        'ZLVDTSens': 2.4883e-06,
        'XLVDTOffset': -0.09,
        'YLVDTOffset': -0.1,
        'ZLVDTOffset': 0.55,
        'YIgain': 4.74,
        'YPgain': '-inf',
        'XIgain': 4.72,
        'XPgain': '-inf',
        'LastScan': 0,
        'XPiezoSens': -1.606e-07,
        'YPiezoSens': 1.8273e-07,
        'ZPiezoSens': 2.8128e-08,
        'XDriveOffset': 0,
        'YDriveOffset': 0,
        'SweepPoints': 480,
        'SecretGain': 0,
        'YSgain': 7.09,
        'XSgain': 7.02,
        'DIsplayLVDTTraces': 0,
        'Initial PhaseOffset': 0,
        'BaseSuffix': 0,
        'SaveImage': 2,
        'ParmChange': 0,
        'ADCgain': 0,
        'OldADCgain': 12,
        'OldScanSize': 'NaN',
        'OldYOffset': 0,
        'OldXOffset': 0,
        'ZIgain': 5.56,
        'ZPgain': '-inf',
        'ZSgain': 0,
        'Initial AmpInvOLS': 4.1332e-08,
        'UpdateCounter': 106,
        'DoMunge': 0,
        'XMungeAlpha': 1,
        'YMungeAlpha': 1,
        'ZMungeAlpha': 1,
        'Has1xACDeflGain': 1,
        'DontChangeXPT': 1,
        'LowNoise': 1,
        'ScreenRatio': 0.75,
        'XDriftRate': 0,
        'YDriftRate': 0,
        'XDriftTotal': 0,
        'YDriftTotal': 0,
        'DriftBIts': 0,
        'DriftCount': 0,
        'Setpoint': 0.0008,
        'Initial FastRatio': 1,
        'Initial SlowRatio': 1,
        'Initial TopLine': 255,
        'Initial BottomLine': 0,
        'ScanStatus': 2,
        'Is1DPlus': 0,
        'DelayUpdate': 0,
        'MarkerRatio': 0,
        'StartLineCount': 0,
        'OldAmplitudeSetpointVolts': 0.8,
        'OldDeflectionSetPointVolts': 0,
        'Initial ScanMode': 0,
        'Interpolate': -1248,
        'TipVoltage': 0,
        'SurfaceVoltage': 0,
        'FreqIgain': 5,
        'FreqPgain': 0,
        'FreqSgain': 0,
        'Initial OrcaGain': 'NaN',
        'FreqGainOn': 0,
        'Initial UserIn0Gain': 1,
        'Initial UserIn1Gain': 1,
        'Initial UserIn2Gain': 1,
        'Initial LateralGain': 1,
        'IsBipolar': 0,
        'FrequencyHack': 0,
        'MagneticField': 'NaN',
        'DriveGainOn': 0,
        'DriveIGain': 50,
        'DrivePGain': 0,
        'DriveSGain': 0,
        'Initial UserIn0Offset': 0,
        'Initial UserIn1Offset': 0,
        'Initial UserIn2Offset': 0,
        'Initial LateralOffset': 0,
        'LastImage': 0,
        'ZoomSize': 0,
        'ZoomXOffset': 2.2506e-06,
        'ZoomYOffset': 4.4481e-06,
        'User0Voltage': 0,
        'User1Voltage': 0,
        'ExtendedZ': 0,
        'DriveAmplitude1': 0.1,
        'DriveFrequency1': 450000,
        'SweepWidth1': 5000,
        'Initial PhaseOffset1': 0,
        'Initial Amp2InvOLS': 2e-08,
        'Initial VoltageParm': 0,
        'MicroscopeID': 4,
        'CapacitanceSens': 1,
        'CapacitanceOffset': 0,
        'CapPhaseOffset': 0,
        'TipBiasOffset': 0,
        'TipHeaterOffset': 0,
        'SurfaceBiasOffset': 0,
        'UseCantHolder': 2,
        'CurrentCantHolder': 2,
        'HasClickRing': 1,
        'XScanDirection': -1,
        'YScanDirection': 1,
        'SaveImageCount': 1,
        'SaveImageLines': 128,
        'Initial OrcaGain2': 'NaN',
        'UpdateSlowWave': 0,
        'FBFilterBW': 5000,
        'CurrentSetpointAmps': 1,
        'CurrentSetpointVolts': 1,
        'OldCurrentSetpointVolts': 1,
        'OldCurrentSetpointAmps': 1,
        'HasFM': 1,
        'LastScanWithdraw': 0,
        'HasLaserRelay': 1,
        'ToggleLaserRelay': 0,
        'Decimation': 39,
        'DARTIGain': 5.5,
        'DARTPGain': 0,
        'SampleHolder': 1,
        'ManualSampleHolder': 0,
        'CypherXPTLock': 1024,
        'XPTLock': 23552,
        'LogFeedback': 0,
        'ARSysScanBusy': 0,
        'SimpleControls': 0,
        'Initial PointsLines': 256,
        'FreeAirAmplitude': 'NaN',
        'FreeAirPhase': 'NaN',
        'FreeAirDeflection': 0.49121,
        'FreeAirDriveAmplitude': 0,
        'ContinualRetune': 0,
        'ForceFilter': 0,
        'TemperatureSens': 1,
        'TemperatureOffset': 0,
        'OffsetBehavior': 0,
        'InputRange': 1,
        'XLVDTSensSlope': 18.352,
        'YLVDTSensSlope': 14.219,
        'XLVDTSensIntercept': 3.7378e-06,
        'YLVDTSensIntercept': 3.9222e-06,
        'HaveWeKilledTheProgressBar': 1,
        'Initial PreScanSetting': 0,
        'ImageFrameTime': 0,
        'FBFilterSelectivity': 0,
        'HasDriveChoke': 1,
        'DriveChokeOn': 0,
        'CFCWFOS': 0,
        'SkewAngle': 1.478e-40,
        'ContinualRetuneTime': 60,
        'DRTimeStamp': 0,
        'CRTimeStamp': 0,
        'Cypher15VPower': 1,
        'HasBlueDrive': 1,
        'BlueDriveOn': 0,
        'blueDriveOffset': 4.5473,
        'blueDriveMaxAmplitude': 2.0342,
        'bDDriveAmplitude': 0.002,
        'bDDriveAmplitude1': 0.002,
        'bDDiffEfficiencyV': 0.0044243,
        'Initial BackPackIn0Gain': 1,
        'Initial BackPackIn0Offset': 0,
        'Initial BackPackIn1Gain': 1,
        'Initial BackPackIn1Offset': 0,
        'Initial BackPackIn2Gain': 1,
        'Initial BackPackIn2Offset': 0,
        'Initial BackPackIn3Gain': 1,
        'Initial BackPackIn3Offset': 0,
        'Initial BackPackIn4Gain': 1,
        'Initial BackPackIn4Offset': 0,
        'bdReCalibrateTime': 1,
        'bdMotorError': 0,
        'ScanModeGUI': 0,
        'FastMapSetpointVolts': 1,
        'FastMapSetpointNewtons': 1.7647e-08,
        'OldFastMapSetpointVolts': 1,
        'OldFastMapSetpointNewtons': 1,
        'Initial FastMapZRate': 299.4,
        'FastMapAmplitude': 4e-07,
        'FastMapFBMethod': 0,
        'FastMapFBPhase': 0,
        'FastMapFBWindow': 10,
        'FastMapAmpSetpointVolts': 0.8,
        'FastMapAmpSetpointMeters': 1e-08,
        'FastMapIndex': 0,
        'QueueProgress': 0,
        'QueueEstimate': 0,
        'QueueExpand': 0,
        'FastMapIGain': 19.557,
        'Initial FastMapScanRate': 0.46781,
        'FastMapBaseLine': 0,
        'blueThermOn': 0,
        'FastMapScanPoints': 256,
        'FastMapScanLines': 256,
        'FastMapOptScanPoints': 256,
        'FastMapOptScanLines': 256,
        'FreqDGain': 0,
        'DriveDGain': 0,
        'ImageSaveExpand': 0,
        'Saturated': 0,
        'FrameCount': 0,
        'SinusoidalOn': 0,
        'bdMaskGain': 1,
        'LastThreadHandle': 527,
        'StartHeaterTemp': 'NaN °C',
        'StartHeaterHumidity': 'NaN',
        'MicroscopeModel': 'Cypher',
        'BlueDriveDissipationInWatts': 0,
        'NapIntegralGain': 10,
        'NapProportionalGain': 0,
        'NapDeflectionSetpointVolts': 0.5,
        'NapAmplitudeSetpointVolts': 0.8,
        'NapDissipationSetpointVolts': 0.2,
        'NapFrequencySetPointHz': 0,
        'NapCurrentSetpointAmps': 1,
        'NapDriveAmplitude': 1,
        'NapDriveFrequency': 72969,
        'NapDriveAmplitude1': 0.1,
        'NapDriveFrequency1': 420000,
        'NapTipVoltage': 3,
        'NapSurfaceVoltage': 0,
        'NapUser0Voltage': 0,
        'NapUser1Voltage': 0,
        'NapHeight': 5e-08,
        'NapStartHeight': 0,
        'NapTime': 0.10016,
        'PotentialIGain': 3,
        'PotentialPGain': 0,
        'PotentialGainOn': 0,
        'ElectricTune': 0,
        'NapCurrentSetpointVolts': 1,
        'PotentialLoopLimit': 9,
        'ClosedNap': 0,
        'NapZIGain': 50,
        'NapZPGain': 0,
        'AutoNapZGain': 1,
        'bDNapDriveAmplitude': 0.002,
        'bDNapDriveAmplitude1': 0.002,
        'Channel2DataType': 'Amplitude',
        'Channel3DataType': 'Phase',
        'Channel4DataType': 'Deflection',
        'Channel5DataType': 'None',
        'Channel6DataType': 'None',
        'Channel7DataType': 'None',
        'Channel8DataType': 'None',
        'UserCalcFunc': 'CalcPhaseFromIQ',
        'UserCalcBFunc': 'CalcAmpFromIQ',
        'FastMap1DataType': 'Height',
        'FastMap2DataType': 'MaxForce',
        'FastMap3DataType': 'Adhesion',
        'FastMap4DataType': 'HertzSphere2P',
        'FastMapCapture': 1,
        'InA': 'FilterOut',
        'InB': 'Ground',
        'InFast': 'ACDefl',
        'InAOffset': 'Ground',
        'InBOffset': 'Ground',
        'InFastOffset': 'Ground',
        'OutXMod': 'Off',
        'OutYMod': 'Off',
        'OutZMod': 'Off',
        'FilterIn': 'Defl',
        'BNCOut0': '$DDSDCOffset0',
        'BNCOut1': 'Output.B',
        'BNCOut2': 'ACDefl',
        'PogoOut': 'Ground',
        'Chip': 'Ground',
        'Shake': 'Ground',
        'Input.Z.Filter.Freq': 641.03,
        'Input.X.Filter.Freq': 641.03,
        'Input.Y.Filter.Freq': 641.03,
        'Input.A.Filter.Freq': 641.03,
        'Input.B.Filter.Freq': 641.03,
        'Input.Fast.Filter.Freq': 641.03,
        'Lockin.0.Filter.Freq': 641.03,
        'Lockin.1.Filter.Freq': 641.03,
        'Cypher.Input.FastA.Filter.Freq': 5000,
        'Cypher.Input.FastB.Filter.Freq': 641.03,
        'Cypher.Input.A.Filter.Freq': 641.03,
        'Cypher.Input.B.Filter.Freq': 641.03,
        'Cypher.Input.C.Filter.Freq': 641.03,
        'Cypher.LVDT.X.Filter.Freq': 2000,
        'Cypher.LVDT.Y.Filter.Freq': 2000,
        'Cypher.LVDT.Z.Filter.Freq': 641.03,
        'Cypher.LockinA.0.Filter.Freq': 641.03,
        'Cypher.LockinA.1.Filter.Freq': 641.03,
        'Cypher.LockinB.0.Filter.Freq': 641.03,
        'Cypher.LockinB.1.Filter.Freq': 641.03,
        'Sample Rate': 1282.1,
        'ThermalDC': 2.3668e-14,
        'ThermalQ': 138.63,
        'ThermalFrequency': 72969,
        'ThermalWhiteNoise': 3.9017e-13,
        'ScaleLog': 3,
        'FitWidth': 20000,
        'ThermalSamples': 1000,
        'ThermalCounter': 127,
        'SectionSamples': 10,
        'SectionCounter': 0,
        'DoThermal': 0,
        'ThermalResolution': 4,
        'ThermalZoom': 0,
        'DoCantTune': 0,
        'ZoomCounter': 200,
        'AutoTuneLow': 50000,
        'AutoTuneHigh': 400000,
        'TargetVoltage': 1,
        'TargetPercent': 0,
        'CheckTuneTraces': 0,
        'ButtonTime': 2088000000000,
        'ShowThermalFit': 1,
        'AppendThermalBit': 8,
        'TuneQ': 100,
        'TuneGain': 0,
        'TunePeakResult': 0.0071096,
        'TuneFreqResult': 368990,
        'TuneQResult': 213.46,
        'ThermalSamplesLimit': 0,
        'TuneFeedback': 0,
        'TuneDDSOutput': 0,
        'TipVoltCenter': 0,
        'TipVoltWidth': 10,
        'ThermalFrequencyRange': 3,
        'AutoTuneLimit': 0,
        'AutoTuneLow1': 364850,
        'AutoTuneHigh1': 547270,
        'TargetVoltage1': 0.1,
        'TargetPercent1': 0,
        'DualACMode': 0,
        'WhichACMode': 0,
        'BackwardsTune': 0,
        'TuneCaptureRate': 500,
        'TuneTime': 0.96,
        'TuneFilter': 500,
        'iDriveOn': 0,
        'FrequencyRatio': 1.1819,
        'DFRTFrequencyCenter': 415370,
        'DFRTFrequencyWidth': 69256,
        'DFRTOn': 0,
        'ThermalCenter': 72969,
        'ThermalWidth': 30000,
        'DFWMAMT': 0,
        'LastXPos': 70,
        'LastYPos': 70,
        'PFMFrequencyLimit': 'inf',
        'TuneLockin': 0,
        'TipHeaterDrive': 0,
        'AdjustSpringConstant': 1,
        'DrawThermalFit': 0,
        'LateralTune': 0,
        'ThermalTemperature': 300.02,
        'UseCypherThermal': 1,
        'UseCypherMultiThread': 0,
        'Illumination': 25,
        'ARCZ': 1,
        'TuneAngle': -90,
        'IsDefaultFrequency': 0,
        'TargetAmplitude': 0,
        'TargetPhase': 90,
        'LossTanAmp1': 0.00055505,
        'LossTanPhase1': -160.55,
        'LossTanAmp2': 0.067638,
        'LossTanPhase2': -179.99,
        'LossTanFreq1': 380740,
        'LossTanFreq2': 0,
        'LossTanQ1': 213.46,
        'LossTanQ2': 0,
        'SecondFrequency': 0,
        'TryFitAgain': 0,
        'HighFrequencyMode': 0,
        'ThermalLever': 'None',
        'DoSader': 0,
        'CustomLeverLength': 0.00024,
        'CustomLeverWidth': 3e-05,
        'CustomLeverFrequency': 70000,
        'ResFreq1': 368990,
        'ResFreq2': 0,
        'GetStartedRoughness': 4,
        'GetStartedOn': 0,
        'GSGainGain': 1,
        'GSSetpointPercent': 50,
        'GSHardness': 0,
        'GSLeverRange': 1,
        'GetStartedMorePanel': 0,
        'GSUseManualThermal': 0,
        'GSMotorHeight': 0,
        'GSStickySample': 0,
        'GSDateTime': 1,
        'BadThermal': 0,
        'YourVariablesStartHere': 0,
        'Log File': 'Stop',
        'Date': '2021-01-07',
        'Seconds': 3692884718.399,
        'Inside Temp': 30.062,
        'ATC Temp0': 4.9438,
        'Outside Temp': 24.833,
        'ATC Temp1': 23.674,
        'ATC Heater Case Temp': 32.072,
        'ATC Exit Temp': 37.019,
        'ATC Case Temp': 23.384,
        'ATC Heater PWM': 23.363,
        'ATC Fan PWM': 100,
        'LaserModule': '10x30 µm Laser',
        'UserCalcUnit': '°',
        'UserIn0Unit': 'V',
        'UserIn1Unit': 'V',
        'UserIn2Unit': 'V',
        'LateralUnit': 'V',
        'ARUserPanelName': '',
        'UserIn0Name': '',
        'UserIn1Name': '',
        'UserIn2Name': '',
        'UserCalcName': '',
        'UserIn0Ab': '',
        'UserIn1Ab': '',
        'UserIn2Ab': '',
        'UserCalcAb': '',
        'UserIn0Force': '',
        'UserIn1Force': '',
        'UserIn2Force': '',
        'UserCalcForce': '',
        'NavLoadPath': '',
        'LastNavLoadPath': '',
        'UserCalcBUnit': '',
        'UserCalcBName': '',
        'UserCalcBForce': '',
        'UserCalcBAb': '',
        'BackPackIn0Unit': 'V',
        'BackPackIn0Name': '',
        'BackPackIn0Ab': '',
        'BackPackIn0Force': '',
        'BackPackIn1Unit': 'V',
        'BackPackIn1Name': '',
        'BackPackIn1Ab': '',
        'BackPackIn1Force': '',
        'BackPackIn2Unit': 'V',
        'BackPackIn2Name': '',
        'BackPackIn2Ab': '',
        'BackPackIn2Force': '',
        'BackPackIn3Unit': 'V',
        'BackPackIn3Name': '',
        'BackPackIn3Ab': '',
        'BackPackIn3Force': '',
        'BackPackIn4Unit': 'V',
        'BackPackIn4Name': '',
        'BackPackIn4Ab': '',
        'BackPackIn4Force': '',
        'DetectorSum': 'Input.x',
        'XSensor': 'Cypher.LVDT.X',
        'YSensor': 'Cypher.LVDT.Y',
        'ZSensor': 'Cypher.LVDT.Z',
        'DDSAmplitude0': '$Lockin.0.Amp',
        'DDSDCOffset0': '$Lockin.DCOffset',
        'DDSFrequency0': '$Lockin.0.Freq',
        'DDSFrequencyOffset0': '$Lockin.0.FreqOffset',
        'DDSPhaseOffset0': '$Lockin.0.PhaseOffset',
        'DDSAmplitude1': '$Lockin.1.Amp',
        'DDSFrequency1': '$Lockin.1.Freq',
        'DDSPhaseOffset1': '$Lockin.1.PhaseOffset',
        'Amplitude': '$Lockin.0.r',
        'Phase': '$Lockin.0.theta',
        'Inputi': '$Lockin.0.i',
        'Inputq': '$Lockin.0.q',
        'Inputi1': '$Lockin.1.i',
        'Inputq1': '$Lockin.1.q',
        'Count': 'Input.Counter0',
        'Count2': 'Input.Counter1',
        'Potential': '$DDSDCOffset0',
        'Amplitude1': '$Lockin.1.r',
        'Phase1': '$Lockin.1.theta',
        'DDSQGain0': '$Lockin.0.QGain',
        'DDSQAngle0': '$Lockin.0.QAngle',
        'DDSFrequencyOffset1': '$Lockin.1.FreqOffset',
        'Deflection': 'Cypher.Input.FastA',
        'Lateral': 'Input.Y',
        'Frequency': '$DDSFrequencyOffset1',
        'LockIn': 'ARC.Lockin',
        'Height': 'Arc.Output.Z',
        'DeflectionValue': '$Deflection',
        'HeightLoop': 'PIDSloop.2',
        'VerDate': 140157,
        'Version': '14.30.157',
        'XopVersion': '20132.34.23.0',
        'OSVersion': 'Windows 7 Professional Service Pack 1 (Build 7601)',
        'IgorFileVersion': '6.3.7.2',
        'BaseName': 'BTFO_DSO_Thick',
        'SkewAngleRead': '89.99999 °',
        'ImageNote': 'sample is at 45 deg to cantilever',
        'TipSerialNumber': '',
        'Planefit 0': 'Flatten 1',
        'PlanefitOrder 0': -1,
        'FlattenOrder 0': 1,
        'Planefit Offset 0': -4.6222e-07,
        'Planefit X Slope 0': 0,
        'Planefit Y Slope 0': 0,
        'Flatten Offsets 0': '-6.83573667e-08,-7.15477443e-08,-7.43942376e-08,-7.69014642e-08,-7.98011151e-08,-8.26604978e-08,-8.56914387e-08,-8.86298962e-08,-9.19689025e-08,-9.74196334e-08,-1.02970046e-07,-1.07450725e-07,-1.13727988e-07,-1.18501363e-07,-1.21702881e-07,-1.24670622e-07,-1.27547746e-07,-1.30237578e-07,-1.32954665e-07,-1.35775979e-07,-1.38676218e-07,-1.41549386e-07,-1.44449541e-07,-1.47473927e-07,-1.50441816e-07,-1.53376481e-07,-1.56052003e-07,-1.58667157e-07,-1.61576049e-07,-1.64238258e-07,-1.67267898e-07,-1.70373907e-07,-1.73106495e-07,-1.76322954e-07,-1.789771e-07,-1.81648179e-07,-1.84738576e-07,-1.87596933e-07,-1.90713392e-07,-1.93607127e-07,-1.96408828e-07,-1.99398437e-07,-2.0200986e-07,-2.05008624e-07,-2.0796619e-07,-2.13309533e-07,-2.19744765e-07,-2.24132595e-07,-2.26875657e-07,-2.30091171e-07,-2.33098301e-07,-2.36113256e-07,-2.38943839e-07,-2.41767018e-07,-2.44651508e-07,-2.47482452e-07,-2.50239164e-07,-2.40969255e-07,-2.41681328e-07,-2.43250534e-07,-2.45173722e-07,-2.47853779e-07,-2.50648964e-07,-2.53572738e-07,-2.5630552e-07,-2.59049696e-07,-2.61907957e-07,-2.64716892e-07,-2.67250455e-07,-2.69999583e-07,-2.72468294e-07,-2.75585295e-07,-2.78508284e-07,-2.81774029e-07,-2.845954e-07,-2.87639361e-07,-2.90951184e-07,-2.94160883e-07,-2.97191555e-07,-3.00418566e-07,-3.03705076e-07,-3.0710822e-07,-3.10824209e-07,-3.14177748e-07,-3.17902231e-07,-3.21151414e-07,-3.24255118e-07,-3.27754377e-07,-3.31759908e-07,-3.35795475e-07,-3.39117615e-07,-3.42181456e-07,-3.45256214e-07,-3.48538659e-07,-3.51873163e-07,-3.5546063e-07,-3.5926938e-07,-3.62700369e-07,-3.66188383e-07,-3.69511536e-07,-3.72691408e-07,-3.75836799e-07,-3.79335962e-07,-3.83056183e-07,-3.87183761e-07,-3.90942404e-07,-3.94753937e-07,-3.9835226e-07,-4.01868696e-07,-4.04994967e-07,-4.07917433e-07,-4.10455496e-07,-4.13144051e-07,-4.15729155e-07,-4.18171154e-07,-4.20751199e-07,-4.2336412e-07,-4.26235553e-07,-4.28984031e-07,-4.31480521e-07,-4.34271841e-07,-4.37030761e-07,-4.39936132e-07,-4.42751956e-07,-4.45692893e-07,-4.48530878e-07,-4.51553772e-07,-4.54312181e-07,-4.56935186e-07,-4.59551386e-07,-4.62233237e-07,-4.6483435e-07,-4.67353056e-07,-4.69877396e-07,-4.72659608e-07,-4.75620405e-07,-4.78646163e-07,-4.81510402e-07,-4.8400599e-07,-4.86882597e-07,-4.89591918e-07,-4.9230249e-07,-4.95344934e-07,-4.9832021e-07,-5.01500284e-07,-5.04444922e-07,-5.07476562e-07,-5.10214496e-07,-5.13045535e-07,-5.15771496e-07,-5.18678624e-07,-5.21665539e-07,-5.27084938e-07,-5.38484428e-07,-5.45483965e-07,-5.4930344e-07,-5.52796396e-07,-5.5609906e-07,-5.59323662e-07,-5.62729078e-07,-5.66390885e-07,-5.68728205e-07,-5.71814329e-07,-5.74557615e-07,-5.77663864e-07,-5.8064858e-07,-5.83304529e-07,-5.85797296e-07,-5.88257886e-07,-5.90694692e-07,-5.9360608e-07,-5.96288423e-07,-5.98924658e-07,-6.02215559e-07,-6.05356168e-07,-6.08174828e-07,-6.10390146e-07,-6.13340881e-07,-6.16059894e-07,-6.19284908e-07,-6.22346308e-07,-6.25259669e-07,-6.28317476e-07,-6.30980835e-07,-6.34809626e-07,-6.39331543e-07,-6.43881127e-07,-6.4750644e-07,-6.50921037e-07,-6.54117821e-07,-6.57033996e-07,-6.59833785e-07,-6.62670642e-07,-6.65469946e-07,-6.68380496e-07,-6.71257439e-07,-6.74143842e-07,-6.77522984e-07,-6.80905571e-07,-6.84326146e-07,-6.87565572e-07,-6.90648853e-07,-6.94554211e-07,-6.98038843e-07,-7.01546351e-07,-7.05397846e-07,-7.08716424e-07,-7.11944189e-07,-7.15210078e-07,-7.18262187e-07,-7.21200491e-07,-7.24287228e-07,-7.27738569e-07,-7.30983668e-07,-7.34225455e-07,-7.37520388e-07,-7.41144744e-07,-7.4456845e-07,-7.48310473e-07,-7.51725649e-07,-7.5508978e-07,-7.58202208e-07,-7.61049085e-07,-7.64141287e-07,-7.67431483e-07,-7.70597799e-07,-7.73989301e-07,-7.77254882e-07,-7.80581459e-07,-7.83497804e-07,-7.86848215e-07,-7.90180236e-07,-7.93587894e-07,-7.97080064e-07,-8.00381064e-07,-8.01420609e-07,-8.05407194e-07,-8.09361456e-07,-8.13099664e-07,-8.16265138e-07,-8.19220937e-07,-8.22263831e-07,-8.25728218e-07,-8.29058287e-07,-8.30749741e-07,-8.33428353e-07,-8.36444301e-07,-8.39582102e-07,-8.42783817e-07,-8.46011903e-07,-8.48866828e-07,-8.51995838e-07,-8.5512312e-07,-8.58522483e-07,-8.62024439e-07,-8.65970638e-07,',
        'Flatten Slopes 0': '0.0679478989,0.0680389352,0.0679695611,0.0679193337,0.0680472975,0.0680356962,0.0680586757,0.0680823668,0.0681749117,0.0682999057,0.068305308,0.0682022398,0.0682380951,0.0681279729,0.0681522284,0.0681760731,0.0681445538,0.0680760965,0.0681035598,0.068128905,0.068152984,0.0682198935,0.0681987191,0.0682041249,0.0681614422,0.0683247751,0.0683312047,0.0682309809,0.0682515323,0.0681657483,0.0682214989,0.0684717046,0.068533648,0.0687663469,0.06857741,0.0687365326,0.06878868,0.0688301304,0.0688197052,0.0687905269,0.0687649672,0.068742822,0.0687899151,0.068799931,0.0685774701,0.0687904751,0.0680245945,0.0687804071,0.0686858873,0.0686952329,0.0686616588,0.0686705963,0.0685872916,0.0685916396,0.0685727016,0.0685638599,0.0685058488,0.0614185503,0.0678076773,0.0682277153,0.0682890325,0.0684183836,0.0686032412,0.068719388,0.0686907233,0.0686688382,0.0686681599,0.0687570539,0.0687214089,0.0687971575,0.0686523857,0.0686017954,0.06842767,0.0684397111,0.0683481631,0.0685114592,0.0686017835,0.0685935141,0.068544588,0.0685315783,0.0685765179,0.0685443297,0.0687726613,0.0688633945,0.0690411429,0.0691189042,0.0691264929,0.0691092851,0.0692387202,0.069350337,0.0693371155,0.0693966191,0.0695103272,0.0696156898,0.0696477687,0.0697383093,0.0697991576,0.0698239079,0.0698562001,0.0698478957,0.0699027625,0.069909127,0.0699749671,0.069928847,0.0698813255,0.0698540479,0.0698858918,0.0697582166,0.0697220591,0.0696387848,0.0695380696,0.0693970612,0.069461491,0.069429215,0.0694520728,0.0694670682,0.0695737267,0.0696582984,0.0696588826,0.0696118457,0.069518087,0.0694238722,0.0694163307,0.0692367088,0.0692566182,0.0693403908,0.069507015,0.0694470964,0.0695316876,0.0695464608,0.069564565,0.0695170681,0.0694996512,0.0695537815,0.0696909697,0.0697363786,0.0699112865,0.0699970944,0.0700853353,0.0700630071,0.0700676663,0.0699068484,0.0698449245,0.0698802038,0.0700675042,0.0700419726,0.069989707,0.0699089669,0.0698926491,0.0698849997,0.0699038763,0.0700604094,0.0699907948,0.0700657329,0.0702646556,0.0702633464,0.0703376961,0.07033527,0.0704320718,0.0701649489,0.0704315762,0.0701049332,0.0703017513,0.0699771493,0.0699940006,0.0701955453,0.0702981186,0.0704405066,0.0704388555,0.070549213,0.0706313737,0.07057596,0.0706035084,0.0706834253,0.0706681496,0.0706349949,0.0704187436,0.070611633,0.0706113974,0.0706764135,0.070648583,0.0706595803,0.0706299394,0.0705866132,0.0707476239,0.0707096712,0.0707294615,0.0706827671,0.0708038823,0.0707275042,0.0707304492,0.0707752274,0.0709356152,0.0709440707,0.0708761864,0.0709675832,0.0709738076,0.0711199707,0.0711973821,0.0714050335,0.0713599359,0.0712004679,0.07126904,0.0710632578,0.0709361632,0.0709818368,0.0711001745,0.0710926298,0.0711869303,0.0711651308,0.0711436475,0.0711612829,0.0712364959,0.0711160815,0.0711361923,0.0712933194,0.0713957975,0.0713329661,0.0715270246,0.0716077195,0.0716543033,0.0717119997,0.0716285477,0.0716516543,0.0716555667,0.071623607,0.0716015617,0.0716462505,0.0716876707,0.0716443622,0.0718001902,0.071831816,0.0719137473,0.0720494556,0.072103402,0.0720760445,0.0721512939,0.0721257116,0.0721043857,0.0719753962,0.0718273742,0.0718004888,0.0719019299,0.0718209094,0.0708812991,0.0717252951,0.0717739732,0.0718862092,0.0720137752,0.0721048911,0.0719990065,0.0720028099,0.0719671013,0.0720442836,0.0721760551,0.0723005386,',
        'Display Offset 0': 0,
        'Display Range 0': 3.4e-09,
        'ColorMap 0': 'VioletOrangeYellow',
        'Planefit 1': 'None',
        'PlanefitOrder 1': -1,
        'FlattenOrder 1': -1,
        'Planefit Offset 1': 1.0815e-11,
        'Planefit X Slope 1': 0,
        'Planefit Y Slope 1': 0,
        'Display Offset 1': 1.0815e-11,
        'Display Range 1': 1.6e-11,
        'ColorMap 1': 'VioletOrangeYellow',
        'Planefit 2': 'None',
        'PlanefitOrder 2': -1,
        'FlattenOrder 2': -1,
        'Planefit Offset 2': 3.773e-08,
        'Planefit X Slope 2': 0,
        'Planefit Y Slope 2': 0,
        'Display Offset 2': 3.773e-08,
        'Display Range 2': 1.7e-08,
        'ColorMap 2': 'VioletOrangeYellow',
        'Planefit 3': 'None',
        'PlanefitOrder 3': -1,
        'FlattenOrder 3': -1,
        'Planefit Offset 3': 110.19,
        'Planefit X Slope 3': 0,
        'Planefit Y Slope 3': 0,
        'Display Offset 3': 110.19,
        'Display Range 3': 170,
        'ColorMap 3': 'VioletOrangeYellow',
        'EndHeaterTemp': 'NaN °C',
        'EndHeaterHumidity': 'NaN',
        'creationDate': 3373459566,
        'modDate': 3692884718,
        'bname': b'BTFO_DSO_Thick0000'}

        metadata = datasets['Channel_000'].original_metadata

        data_labels = [['x (m)', 'y (m)'],
                        ['x (m)', 'y (m)'],
                        ['x (m)', 'y (m)'],
                        ['x (deg)', 'y (deg)']]

        for key in original_metadata:
            if original_metadata[key] =='inf' or original_metadata[key] =='-inf': 
                pass
            else:
                assert original_metadata[key] == metadata[key], "Metadata incorrect for key {}, should be {} " \
                        "but was read as {}".format(key, original_metadata[key], metadata[key])

            
        #for ind in range(len(datasets)):
        for ind,key in enumerate(datasets):
            assert type(datasets[key])== sidpy.sid.dataset.Dataset, "Dataset No. {} not read in as sidpy dataset" \
                    "but was instead read in as {}".format(ind, type(datasets[key]))

            assert datasets[key].labels == data_labels[ind], "Dataset {} label should be a {} but " \
                                                      "is instead {}".format(ind,data_labels[ind], datasets[key].labels)

            assert datasets[key].shape==(256, 256), "Dataset[{}] is of size (256,256) but was read in as {}".format(ind, datasets[key].shape)
            assert type(datasets[key]._axes[0]) == sidpy.sid.dimension.Dimension, "Dataset should have dimension type " \
                                           "of sidpy Dimension, but is instead {}".format(type(datasets[key]._axes))

            assert datasets[key].data_descriptor == data_descriptors[ind], "Dataset {} " \
            "should have descriptor {} but instead has descriptor {}".format(ind, data_descriptors[ind], datasets[key].data_descriptor)
        
        os.remove(file_path)
